@@ -1,4 +1,4 @@
-# ZestyParser 0.5.0 -- Parses in Python zestily
+# ZestyParser 0.5.1 -- Parses in Python zestily
 # Copyright (C) 2006 Adam Atlas
 # 
 # This program is free software; you can redistribute it and/or
@@ -65,9 +65,6 @@ class ZestyParser:
 		if not hasattr(tokens, '__iter__'):
 			tokens = (tokens,)
 		for t in tokens:
-			if t.__class__ is str:
-				t = self.tokens[t]
-
 			oldCursor = self.cursor
 			try:
 				r = t(self, oldCursor)
@@ -80,8 +77,6 @@ class ZestyParser:
 		return None
 	
 	def skip(self, t):
-		if t.__class__ is str:
-			t = self.tokens[t]
 		oldCursor = self.cursor
 		try:
 			t(self, oldCursor)
@@ -91,7 +86,7 @@ class ZestyParser:
 			else: return False
 		return True
 	
-	def iter(self, tokens, skip=None): return ParserIterator(tokens, self, skip)
+	def iter(self, tokens, skip=None, until=None): return ParserIterator(tokens, self, skip, until)
 	
 	def coord(self, loc=None):
 		if loc is None: loc = self.cursor
@@ -100,15 +95,18 @@ class ZestyParser:
 		return (row + 1, col)
 
 class ParserIterator:
-	def __init__(self, tokens, parser, skip=None):
-		self.tokens, self.parser, self.skip = tokens, parser, skip
+	def __init__(self, tokens, parser, skip=None, until=None):
+		self.tokens, self.parser, self.skip, self.until = tokens, parser, skip, until
 	
 	def __iter__(self): return self
 	
 	def next(self):
 		if self.skip:
 			self.parser.skip(self.skip)
+		if self.until and self.parser.skip(self.until[0]):
+			raise StopIteration
 		r = self.parser.scan(self.tokens)
 		if self.parser.last is None:
-			raise StopIteration
+			if self.until: raise ParseError(self.parser, self.until[1])
+			else: raise StopIteration
 		return r
